/* Copyright (C) 1992-1998 The Geometry Center
 * Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips
 *
 * This file is part of Geomview.
 * 
 * Geomview is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * Geomview is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with Geomview; see the file COPYING.  If not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA, or visit http://www.gnu.org.
 */

#if defined(HAVE_CONFIG_H) && !defined(CONFIG_H_INCLUDED)
#include "config.h"
#endif

static char copyright[] = "Copyright (C) 1992-1998 The Geometry Center\n\
Copyright (C) 1998-2000 Stuart Levy, Tamara Munzner, Mark Phillips";


/* Authors: Charlie Gunn, Stuart Levy, Tamara Munzner, Mark Phillips */

#include "bezierP.h"

BBox *
BezierBound( bezier, T )
    register Bezier *bezier;
    Transform T;
{
    register float *p;
    register int v, n;
    HPoint3 min, max;

    /* only support 3 and 4 dimn patches */
    if(bezier->dimn == 4) {
	/* It may not make sense to find the bbox of these 4-D control points.
	 * Dice it into a real mesh and take the bbox of that.
	 */
	if(bezier->flag & BEZ_REMESH ||
	   bezier->mesh == NULL || bezier->mesh->p == NULL) {
	    if(BezierReDice(bezier) == NULL)
		return NULL;		/* Oh no */
	}
	return MeshBound( bezier->mesh, T );
    }

    if (bezier->dimn != 3) {
       GeomError(0,"BezierBound: invalid dimension %d",bezier->dimn);
       return(NULL);
    }

    n = (bezier->degree_u + 1) * (bezier->degree_v + 1);

    p = bezier->CtrlPnts;
    Pt3Transform( T, p, &min );
    min.w = 1;
    max = min;
    for( v = n; --v > 0; ) {
	Point3 t;

	p += 3;
	Pt3Transform( T, p, &t );
	if(min.x > t.x) min.x = t.x;
	else if(max.x < t.x) max.x = t.x;
	if(min.y > t.y) min.y = t.y;
	else if(max.y < t.y) max.y = t.y;
	if(min.z > t.z) min.z = t.z;
	else if(max.z < t.z) max.z = t.z;
    }

    return (BBox *) GeomCCreate (NULL, BBoxMethods(), CR_MIN, &min, CR_MAX, &max, NULL);
}
